<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 *
 * With the purchase or the installation of the software in your application
 *
 * you accept the licence agreement.
 *
 *
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    Banca Sella S.p.A
 *  @copyright 1996-2024 Banca Sella S.p.A
 *  @license   LICENSE.txt
 */

/*if (!defined('_PS_VERSION_')) {
    exit;
}*/

class APPpago extends PaymentModule
{
    private $_minInstallmentsNumber;
    private $_maxInstallmentsNumber;
    private $_minInstallmentsAmount;
    private $_maxInstallmentsAmount;
    private $_minInstallmentsQuote = 20;
    private $_excludedCategories = [];

    public function __construct()
    {
        // set name
        $this->name = 'apppago';
        // set tab
        $this->tab = 'payments_gateways';
        // set version
        $this->version = '1.4.2';
        // set author
        $this->author = 'Banca Sella';
        // set need_instance
        $this->need_instance = 0;
        // set dependencies
        $this->dependencies = [];
        // set ps_versions_compliancy
        $this->ps_versions_compliancy = ['min' => '1.6', 'max' => _PS_VERSION_];
        // set bootstrap
        $this->bootstrap = true;
        // call parent constructor
        parent::__construct();
        // set displayName
        $this->displayName = 'AppPago';
        // set description
        $this->description = 'AppPago Payment Gateway';
        // set confirmUninstall
        $this->confirmUninstall = 'Are you sure you want to uninstall?';
        // enable configuration
        $this->controllers = ['config'];

        $this->initConfig();
    }

    public function initConfig()
    {
        $this->_minInstallmentsNumber = empty(Configuration::get('APPPAGO_RANGE_INSTALLMENTS_MIN')) ? 2 : Configuration::get('APPPAGO_RANGE_INSTALLMENTS_MIN');
        $this->_maxInstallmentsNumber = empty(Configuration::get('APPPAGO_RANGE_INSTALLMENTS_MAX')) ? 24 : Configuration::get('APPPAGO_RANGE_INSTALLMENTS_MAX');
        $this->_minInstallmentsAmount = empty(Configuration::get('APPPAGO_MIN_INSTALLMENTS')) ? 60 : Configuration::get('APPPAGO_MIN_INSTALLMENTS');
        $this->_maxInstallmentsAmount = empty(Configuration::get('APPPAGO_MAX_INSTALLMENTS')) ? 3000 : Configuration::get('APPPAGO_MAX_INSTALLMENTS');
        $this->_excludedCategories = $this->getSerializedConfiguration('APPPAGO_EXCLUDED_CATEGORIES');
    }

    public function install()
    {
        if (!parent::install() or !$this->registerHook('paymentOptions') or !$this->registerHook('paymentReturn') or !$this->registerHook('displayProductAdditionalInfo')) {
            return false;
        }

        return true;
    }

    // uninstall() method
    public function uninstall()
    {
        // call parent uninstall method
        if (!parent::uninstall()) {
            return false;
        }

        return true;
    }

    private function validateConfig()
    {
        $merchantCode = Tools::getValue('merchantCode');
        $vat = Tools::getValue('vat');
        /*$min_amount = Tools::getValue('min_amount');
        $max_amount = Tools::getValue('max_amount');
        $installments_type = Tools::getValue('installments_type');
        $fixed_installments_number = Tools::getValue('fixed_installments_number');
        $range_installments_min = Tools::getValue('range_installments_min');
        $range_installments_max = Tools::getValue('range_installments_max');
        $select_installments_list = Tools::getValue('select_installments_list');*/

        if (empty($merchantCode)) {
            $errors[] = 'Il campo "Codice Merchant" è obbligatorio';
        }

        if (empty($vat)) {
            $errors[] = 'Il campo "VAT" è obbligatorio';
        }

        /*if (empty($min_amount) || !is_numeric($min_amount) || $min_amount < $this->_minInstallmentsAmount) {
            $errors[] = 'Il campo "Importo Minimo" è obbligatorio e deve essere maggiore di ' . $this->_minInstallmentsAmount;
        }

        if (empty($max_amount) || !is_numeric($max_amount) || $max_amount > $this->_maxInstallmentsAmount) {
            $errors[] = 'Il campo "Importo Massimo" è obbligatorio e deve essere minore di ' . $this->_maxInstallmentsAmount;
        }

        if ($installments_type === 'fixed' && (empty($fixed_installments_number) || $fixed_installments_number < $this->_minInstallmentsNumber || $fixed_installments_number > $this->_maxInstallmentsNumber)) {
            $errors[] = 'Il campo "Numero Rate Fisso" è obbligatorio e deve essere compreso tra ' . $this->_minInstallmentsNumber . ' e ' . $this->_maxInstallmentsNumber;
        }

        if ($installments_type === 'range'
            && (empty($range_installments_min) || empty($range_installments_max)
            || $range_installments_min < $this->_minInstallmentsNumber || $range_installments_max > $this->_maxInstallmentsNumber)
        ) {
            $errors[] = 'I campi "Numero Rate Minimo" e "Numero Rate Massimo" sono obbligatori e devono essere compresi tra ' . $this->_minInstallmentsNumber . ' e ' . $this->_maxInstallmentsNumber;
        }

        if ($installments_type === 'select' && empty($select_installments_list)) {
            $errors[] = 'Il campo "Numero Rate Selezionabili" è obbligatorio';
        }*/

        if(empty($errors)) {
            return true;
        }
        $this->_errors = $errors;
        return false;
    }

    public function postProcess()
    {
        // names are the same as the ones in config.xml
        $merchantCode = Tools::getValue('merchantCode');
        $testingMode = Tools::getValue('testingMode');
        $vat = Tools::getValue('vat');
        $api_key = bin2hex(random_bytes(16));
        $min_amount = Tools::getValue('min_amount');
        $max_amount = Tools::getValue('max_amount');
        // cast to string iban_allowed and company_allowed
        $iban_allowed = (string) Tools::getValue('iban_allowed') ? '1' : '0';
        $company_allowed = (string) Tools::getValue('company_allowed') ? '1' : '0';
        $installments_type = Tools::getValue('installments_type');
        $excluded_categories = Tools::getValue('excluded_categories') ?? [];

        // Campi dinamici
        $fixed_installments_number = '';
        $range_installments_min = '';
        $range_installments_max = '';
        $select_installments_list = [];

        if ($installments_type === 'fixed') {
            $fixed_installments_number = Tools::getValue('fixed_installments_number');
        } elseif ($installments_type === 'range') {
            $range_installments_min = Tools::getValue('range_installments_min');
            $range_installments_max = Tools::getValue('range_installments_max');
        } elseif ($installments_type === 'select') {
            $select_installments_list = Tools::getValue('select_installments_list');
        }

        $base_url = 'https://secure.sellapersonalcredit.it/SPCPlugAppPago/api/config';

        $postFields =
            [
                'platform' => 'prestashop',
                'version' => $this->version,
                'host' => $_SERVER['HTTP_HOST'],
                'merchant_code' => $merchantCode,
                'vat' => $vat,
                'api_key' => $api_key,
                'max_amount' => $max_amount,
                'min_amount' => $min_amount,
                'installments_type' => $installments_type,
                'fixed_installments_number' => $fixed_installments_number,
                'range_installments_min' => $range_installments_min,
                'range_installments_max' => $range_installments_max,
                'select_installments_list' => $select_installments_list,
                'iban_allowed' => $iban_allowed,
                'company_allowed' => $company_allowed,
            ];

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $base_url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postFields));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        $response = curl_exec($ch);

        curl_close($ch);

        $decodedResponse = json_decode($response, true);

        if (isset($decodedResponse['status']) && $decodedResponse['status'] === 'ok') {
            // names correspond to the ones in database table APPpago
            Configuration::updateValue('APPPAGO_TESTING_MODE', $testingMode);
            Configuration::updateValue('APPPAGO_MERCHANT_CODE', $merchantCode);
            Configuration::updateValue('APPPAGO_VAT', $vat);
            Configuration::updateValue('APPPAGO_API_KEY', $api_key);
            Configuration::updateValue('APPPAGO_MIN_INSTALLMENTS', $min_amount);
            Configuration::updateValue('APPPAGO_MAX_INSTALLMENTS', $max_amount);
            Configuration::updateValue('APPPAGO_INSTALLMENTS_TYPE', $installments_type);
            Configuration::updateValue('APPPAGO_FIXED_INSTALLMENTS_NUMBER', $fixed_installments_number);
            Configuration::updateValue('APPPAGO_RANGE_INSTALLMENTS_MIN', $range_installments_min);
            Configuration::updateValue('APPPAGO_RANGE_INSTALLMENTS_MAX', $range_installments_max);
            Configuration::updateValue('APPPAGO_SELECT_INSTALLMENTS_LIST', @serialize($select_installments_list));
            Configuration::updateValue('APPPAGO_EXCLUDED_CATEGORIES', @serialize($excluded_categories));
            Configuration::updateValue('APPPAGO_IBAN_ALLOWED', $iban_allowed);
            Configuration::updateValue('APPPAGO_COMPANY_ALLOWED', $company_allowed);
            $this->_confirmations[] = 'Configurazione salvata con successo';
        } else {
            $this->_errors[] = 'Errore durante la configurazione del modulo. (La configurazione non è stata salvata): ' . $decodedResponse['error'];
        }
    }

    public function getContent()
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST' && $this->validateConfig()) {
            $this->postProcess();
        }

        if (count($this->_confirmations) > 0) {
            // Assegna le conferme a Smarty per la visualizzazione
            $this->context->smarty->assign('confirmations', $this->_confirmations);
        }

        if (count($this->_errors) > 0) {
            // Assegna gli errori a Smarty per la visualizzazione
            $this->context->smarty->assign('errors', $this->_errors);
        }

        $this->initConfig();

        $configArray = [
            'testingMode' => Configuration::get('APPPAGO_TESTING_MODE'),
            'merchantCode' => Configuration::get('APPPAGO_MERCHANT_CODE'),
            'vat' => Configuration::get('APPPAGO_VAT'),
            'api_key' => Configuration::get('APPPAGO_API_KEY'),
            'min_amount' => Configuration::get('APPPAGO_MIN_INSTALLMENTS'),
            'max_amount' => Configuration::get('APPPAGO_MAX_INSTALLMENTS'),
            'installments_type' => Configuration::get('APPPAGO_INSTALLMENTS_TYPE'),
            'fixed_installments_number' => Configuration::get('APPPAGO_FIXED_INSTALLMENTS_NUMBER'),
            'range_installments_min' => Configuration::get('APPPAGO_RANGE_INSTALLMENTS_MIN'),
            'range_installments_max' => Configuration::get('APPPAGO_RANGE_INSTALLMENTS_MAX'),
            'select_installments_list' => $this->getSerializedConfiguration('APPPAGO_SELECT_INSTALLMENTS_LIST'),
            'excluded_categories' => $this->getSerializedConfiguration('APPPAGO_EXCLUDED_CATEGORIES'),
            'iban_allowed' => Configuration::get('APPPAGO_IBAN_ALLOWED'),
            'company_allowed' => Configuration::get('APPPAGO_COMPANY_ALLOWED'),
        ];


        $this->context->smarty->assign($configArray);

        $tree = new HelperTreeCategories('excludedCategoriesTree', 'Seleziona le categorie da escludere');
        $tree->setTemplate('tree_associated_categories.tpl')
            ->setRootCategory(Category::getRootCategory()->id)
            ->setUseCheckBox(true)
            ->setInputName('excluded_categories')
            ->setSelectedCategories(is_array($configArray['excluded_categories']) ? $configArray['excluded_categories'] : []);

        $this->context->smarty->assign([
            'excludedCategoriesTree' => $tree->render(),
            'minInstallmentsNumber' => $this->_minInstallmentsNumber,
            'maxInstallmentsNumber' => $this->_maxInstallmentsNumber,
        ]);

        return $this->display(__FILE__, 'views/templates/admin/config.tpl');
    }

    public function getPaymentOptions()
    {
        if (!$this->active) {
            return;
        }

        // URL per l'azione del form
        $formAction = $this->context->link->getModuleLink($this->name, 'validation', [], true);

        $errors = '';
        $apppago_cf = '';
        if (isset($this->context->cookie->redirect_errors)) {
            $errors = $this->context->cookie->redirect_errors;
            $apppago_cf = $this->context->cookie->apppago_cf;
            // delete old messages
            unset($this->context->cookie->redirect_errors);
            unset($this->context->cookie->apppago_cf);
        }

        // Assegna l'URL all'azione alla variabile del template
        $this->smarty->assign([
            'action' => $formAction,
            'apppago_errors' => $errors,
            'apppago_cf' => $apppago_cf,
        ]);

        // Carica il template del form
        $paymentForm = $this->fetch('module:apppago/views/templates/front/payment_options_priv.tpl');

        $logo_path = _PS_MODULE_DIR_ . $this->name . '/Apppago_small.webp';
        // Crea un nuovo oggetto PaymentOption
        $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $newOption->setModuleName($this->displayName)
            ->setCallToActionText('Paga in comode quote con ')
            ->setAction($formAction)
            ->setForm($paymentForm);
        $newOption->setLogo(Media::getMediaPath($logo_path));

        return [$newOption];
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        $cart = $params['cart'];

        // escludo il gateway se il totale dell'ordine è inferiore o superiore ai limiti impostati
        $order_total = $cart->getOrderTotal();

        if (!$this->isInstallmentAmountValid(200)) {
            return;
        }

        $excludedCategories = $this->_excludedCategories;
        if(!empty($excludedCategories)) {
            foreach ($cart->getProducts() as $product) {
                if (in_array($product['id_category_default'], $excludedCategories)) {
                    return;
                }
            }
        }

        return $this->getPaymentOptions($params);
    }

    public function hookDisplayProductAdditionalInfo($params)
    {
        $toPrint = 0;
        // $price = preg_replace("/[^-0-9\.]/","", $params['product']['price']);
        $price = $params['product']['rounded_display_price'];

        if ($this->isInstallmentAmountValid($price)) {

            $excludedCategories = $this->_excludedCategories;
            if(!empty($excludedCategories) && in_array($params['product']['id_category_default'], $excludedCategories)) {
                    return '';
            }

            $installmentNumbers = $this->getInstallmentNumbersByType();

            /** @var [installmentAmount => float, installmentNumber => int] $installmentsResult */
            $installmentsResult = $this->calculateInstallmentAmount($installmentNumbers, $price);
            if(empty($installmentsResult)) {
                return '';
            }

            return $this->fetch('module:apppago/views/templates/front/widget.tpl', [
                'installmentAmount' => number_format($installmentsResult['installmentAmount'], 2, '.', ''),
                'installmentNumbers' => $installmentsResult['installmentNumber'],
                'imgsrc' => _PS_MODULE_DIR_ . $this->name . '/apppago.webp'
            ]);
        }
    }

    public function isInstallmentAmountValid($installmentAmount) {
        return $installmentAmount >= $this->_minInstallmentsAmount && $installmentAmount <= $this->_maxInstallmentsAmount;
    }

    /**
     * Ordina l'array delle mensilità possibili dal più grande e poi cicla per calcolare su ognuno il valore della rata
     * mensile: se è maggiore di APPPAGO_MIN_AMOUNT_INSTALLMENTS ritorna il valore, altrimenti calcola sul numero di mensilità
     * successiva. Se nessuna rata calcolata soddisfa il requisito ritorna false
     *
     * @param $installmentsNumberOptions array contenente i numeri di mensilità possibili (3,4,5)
     * @param $price
     * @return array
     */
    public function calculateInstallmentAmount($installmentsNumberOptions, $price) {
        rsort($installmentsNumberOptions, SORT_NUMERIC);
        foreach ($installmentsNumberOptions as $installmentNumber) {
            $installmentAmount = ceil(($price / $installmentNumber) * 100) / 100;
            if ($installmentAmount >= $this->_minInstallmentsQuote) {
                return ['installmentAmount' => $installmentAmount, 'installmentNumber' => $installmentNumber] ;
            }
        }

        return [];
    }

    /**
     * @param $installmentsType
     * @return array|mixed
     */
    public function getInstallmentNumbersByType()
    {
        $installmentsType = Configuration::get('APPPAGO_INSTALLMENTS_TYPE');
        $installmentNumbers = [];
        if ($installmentsType == 'fixed') {
            $installmentNumbers[] = Configuration::get('APPPAGO_FIXED_INSTALLMENTS_NUMBER');
        } else if ($installmentsType == 'range') {
            $installmentNumbers = range(Configuration::get('APPPAGO_RANGE_INSTALLMENTS_MIN'), Configuration::get('APPPAGO_RANGE_INSTALLMENTS_MAX'));
        } else if ($installmentsType == 'select') {
            $installmentNumbers = $this->getSerializedConfiguration('APPPAGO_SELECT_INSTALLMENTS_LIST');
        } else {
            $installmentNumbers[] = $this->_maxInstallmentsNumber;
        }
        return $installmentNumbers;
    }

    public function getSerializedConfiguration($key, $default = [])
    {
        $value = Configuration::get($key);
        if ($value) {
            return @unserialize($value);
        }
        return $default;
    }
}
